#!/bin/sh
###############################################################################
#
#          DELL Inc. PROPRIETARY INFORMATION
# This software is supplied under the terms of a license agreement or
# nondisclosure agreement with Dell Computer Corporation and may not
# be copied or disclosed except in accordance with the terms of that
# agreement.
#
# Copyright (c) 2000-2003 Dell Inc. All Rights Reserved.
#
# Module Name:
#
#   iomsa.sh
#
# Abstract/Purpose:
#
#   Shell script to install Server Administrator RPMs
#
# Environment:
#
#   Linux
#
###############################################################################

PATH=./:/sbin:/usr/sbin:/bin:/usr/bin:${PATH}

SCRIPT_NAME=`echo $0 | sed s!^./!!`
RPM_VERSION=`rpm --version | awk '{print $3}'`

SA_PACKAGE_NAME="ServerAdministrator"
SAIS_PACKAGE_NAME="dellomsa"
SADD_PACKAGE_NAME="dellomsa-drivers"

SA_RPM_NAME="${SA_PACKAGE_NAME}-1.0-0"
SA_RPM_FILE_NAME="${SA_PACKAGE_NAME}-1.0-0.i386.rpm"
SAIS_RPM_NAME="${SAIS_PACKAGE_NAME}-4.120-3924"
SAIS_RPM_FILE_NAME="${SAIS_PACKAGE_NAME}-4.120-3924.i386.rpm"
SADD_RPM_NAME="${SADD_PACKAGE_NAME}-4.120-3924"
SADD_RPM_FILE_NAME="${SADD_PACKAGE_NAME}-4.120-3924.i386.rpm"
SAIS_VERSION_TMP="4.120"
SAIS_VERSION=`echo ${SAIS_VERSION_TMP} | sed "s/\([0-9]\)\([.]\)\([0-9].*\)\([0-9]\)$/\1\2\3.\4/g"`
DCUPDB="dcupdb32"

LOG_FILE="/var/tmp/iomsa.log"
TMP_FILE="/var/tmp/iomsa.tmp"

PARAM_ID_INSTALLATION_RESULT=1
PARAM_ID_INSTALL_ERR_MSG_ID=2
PARAM_ID_UNINSTALL_STRING=5
PARAM_ID_INSTALLED_VERSION=8
PARAM_ID_OMA_DISPLAY_STRING=9

MSG_ID_SYSTEM_NOT_SUPPORTED=1
MSG_ID_NO_KERNEL_SOURCE=50
MSG_ID_KERNEL_NOT_SUPPORTED=51
MSG_ID_UNEXPECTED_ERROR=54

VALUE_PASS="pass"
VALUE_FAIL="fail"
VALUE_SAIS_DESCRIPTION="Instrumentation Service"
if [ ${RPM_VERSION} = 4.0.2 ];
then
	VALUE_UNINST_CMD="rpm -e ${SAIS_PACKAGE_NAME} ${SADD_PACKAGE_NAME} ${SA_PACKAGE_NAME}"
else
	VALUE_UNINST_CMD="rpm -e ${SA_PACKAGE_NAME} ${SADD_PACKAGE_NAME} ${SAIS_PACKAGE_NAME}"
fi


###############################################################################
#
#   Syntax:   iomsa_display_msg <message>
#   Function: Display a message with the script name
#
###############################################################################
function iomsa_display_msg
{
	echo "${SCRIPT_NAME}: $1"
}


###############################################################################
#
#   Syntax:   iomsa_write_status <key> <value>
#   Function: Write install status
#
###############################################################################
function iomsa_write_status
{
	${DCUPDB} /update "$1" "$2"
}


#------------------------------------------------------------------------------
# Perform sanity checks
#------------------------------------------------------------------------------

rm -f ${LOG_FILE}
rm -f ${TMP_FILE}

# If the caller has set CDROM_DIR to a valid directory, go to that directory
if [ ! -z ${CDROM_DIR} ];
then
	if [ -d ${CDROM_DIR} ];
	then
		cd ${CDROM_DIR}
	fi
fi

# Make sure we have the RPMs
if [ ! -f ${SADD_RPM_FILE_NAME} ];
then
	iomsa_display_msg "${SADD_RPM_FILE_NAME} not found"
	exit 2
fi
if [ ! -f ${SAIS_RPM_FILE_NAME} ];
then
	iomsa_display_msg "${SAIS_RPM_FILE_NAME} not found"
	exit 3
fi
if [ ! -f ${SA_RPM_FILE_NAME} ];
then
	iomsa_display_msg "${SA_RPM_FILE_NAME} not found"
	exit 4
fi

#------------------------------------------------------------------------------
# We must do the following to upgrade Server Agent 4.2 or 4.3:
#   1. Save the Server Agent INI files to the proper directory
#   2. Remove the Server Agent 4.2 or 4.3 RPMs
#   3. Install the new Server Administrator RPMs
#
# The RPM upgrade feature is used to upgrade Server Agent 4.4 and later.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Check for Server Agent 4.2 and 4.3
#------------------------------------------------------------------------------
if [ ${HOSTTYPE} != "ia64" ];
then
	OLD_OMSA_INI_DIR="/usr/lib/dell/openmanage/dellomsa/ini"
	INI_SAVE_DIR="/var/omsa/ini"
	INI_FILE_LIST="dccoo32.ini dcprv32.ini dcpub32.ini"

	# Get list of currently installed RPMs
	rpm -qa >${TMP_FILE} 2>&1

	#--------------------------------------------------------------------------
	# See if the Server Agent 4.2 or 4.3 is installed
	#--------------------------------------------------------------------------
	RPM_NAME=`grep ${SAIS_PACKAGE_NAME}-4.[23] ${TMP_FILE}`
	if [ ! -z "${RPM_NAME}" ];
	then
		# Save the Server Agent INI files to the directory that the
		# Server Administrator Instrumentation RPM will check.
		# The Server Administrator Instrumentation RPM will restore
		# these INI files and clean up the directory when it is installed below.
		mkdir -p ${INI_SAVE_DIR}
		for INI_FILE in ${INI_FILE_LIST}
		do	
			cp -p ${OLD_OMSA_INI_DIR}/${INI_FILE} ${INI_SAVE_DIR}
		done

		# Remove the Server Agent RPM
		rpm -e ${RPM_NAME}
	fi

	#--------------------------------------------------------------------------
	# See if the Server Agent 4.2 or 4.3 Device Drivers RPM is installed
	#--------------------------------------------------------------------------
	RPM_NAME=`grep ${SADD_PACKAGE_NAME}-4.[23] ${TMP_FILE}`
	if [ ! -z "${RPM_NAME}" ];
	then
		# Remove the Device Drivers RPM
		rpm -e ${RPM_NAME}
	fi
fi

#------------------------------------------------------------------------------
# Install the Device Drivers RPM
#------------------------------------------------------------------------------

# Upgrade to needed version of Device Drivers RPM
# Use force option so that upgrade can be used to fix corrupted installations
rpm -U --force ${SADD_RPM_FILE_NAME} >${LOG_FILE} 2>&1
if [ $? != 0 ];
then
	# Get Device Drivers RPM error code
	SADD_ERROR=`grep "ERROR" ${LOG_FILE} | sed 's/.*ERROR //'`
	case "${SADD_ERROR}"
	in
		40) MSG_ID=${MSG_ID_SYSTEM_NOT_SUPPORTED} ;;

		41) MSG_ID=${MSG_ID_NO_KERNEL_SOURCE} ;;

		42) MSG_ID=${MSG_ID_KERNEL_NOT_SUPPORTED} ;;

		43) MSG_ID=${MSG_ID_KERNEL_NOT_SUPPORTED} ;;

		44) MSG_ID=${MSG_ID_KERNEL_NOT_SUPPORTED} ;;

		* ) MSG_ID="${MSG_ID_UNEXPECTED_ERROR};${LOG_FILE}" ;;
	esac

	# Remove the Device Drivers RPM to clean up RPM database
	rpm -e ${SADD_RPM_NAME} >/dev/null 2>&1

	# Ouput install status
	iomsa_write_status ${PARAM_ID_INSTALLATION_RESULT} "${VALUE_FAIL}"
	iomsa_write_status ${PARAM_ID_INSTALL_ERR_MSG_ID}  "${MSG_ID}"

	rm -f ${TMP_FILE}
	exit 21
fi

#------------------------------------------------------------------------------
# Install the Instrumentation and Server Administrator RPMs
#------------------------------------------------------------------------------

# Upgrade to needed version of RPMs
# Use force option so that upgrade can be used to fix corrupted installations
rpm -U --force ${SAIS_RPM_FILE_NAME} ${SA_RPM_FILE_NAME} >>${LOG_FILE} 2>&1

#------------------------------------------------------------------------------
# Ouput install status
#------------------------------------------------------------------------------
iomsa_write_status ${PARAM_ID_INSTALLATION_RESULT} "${VALUE_PASS}"
iomsa_write_status ${PARAM_ID_OMA_DISPLAY_STRING}  "${VALUE_SAIS_DESCRIPTION}"
iomsa_write_status ${PARAM_ID_INSTALLED_VERSION}   "${SAIS_VERSION}"
iomsa_write_status ${PARAM_ID_UNINSTALL_STRING}    "${VALUE_UNINST_CMD}"

#check for input parameters
#one of them might be to disable SNMP sets
if [ $1 = "AllowSNMPSets=FALSE" ];
then
	DELLOMSA_INI_DIR="/usr/lib/dell/openmanage/omsa/ini"
	INI_FILE=${DELLOMSA_INI_DIR}/dcpub32.ini
	INI_FILE_TMP=${DELLOMSA_INI_DIR}/snmp.tmp
	TMP0=/tmp/$$.0
	egrep -i "\[SNMP Configuration\]" ${INI_FILE} >/dev/null
	if [ $? != 0 ];
	then
        	echo "[SNMP Configuration]" >>${INI_FILE}
        	echo "AllowSetRequests=FALSE" >>${INI_FILE}
	else
        	egrep -iv "^AllowSetRequests=" ${INI_FILE} >${INI_FILE_TMP}
        	mv -f ${INI_FILE_TMP} ${INI_FILE}
        	echo -e "s/\(^\[SNMP Configuration\]$\)/\\\1/ \\nt do_test \\nb end_test \\n: do_test" >${TMP0}
        	echo "a \\" >>${TMP0}
        	echo "AllowSetRequests=FALSE" >>${TMP0}
        	echo ": end_test" >>${TMP0}
        	sed -f ${TMP0} ${INI_FILE} >${INI_FILE_TMP}
        	mv -f ${INI_FILE_TMP} ${INI_FILE}
	fi
	rm -f ${TMP0}
	rm -f ${INI_FILE_TMP}
fi

if [ $1 = "AllowSNMPSets=TRUE" ];
then
	DELLOMSA_INI_DIR="/usr/lib/dell/openmanage/omsa/ini"
	INI_FILE=${DELLOMSA_INI_DIR}/dcpub32.ini
	INI_FILE_TMP=${DELLOMSA_INI_DIR}/snmp.tmp
	egrep -i "\[SNMP Configuration\]" ${INI_FILE} >/dev/null
	if [ $? = 0 ];
	then
        	egrep -iv "^AllowSetRequests=" ${INI_FILE} >${INI_FILE_TMP}
        	mv -f ${INI_FILE_TMP} ${INI_FILE}
    fi
	rm -f ${INI_FILE_TMP}
fi



#------------------------------------------------------------------------------
# The End
#------------------------------------------------------------------------------
rm -f ${LOG_FILE}
rm -f ${TMP_FILE}
exit 0

