#!/bin/sh
###############################################################################
#
#          DELL Inc. PROPRIETARY INFORMATION
# This software is supplied under the terms of a license agreement or
# nondisclosure agreement with Dell Computer Corporation and may not
# be copied or disclosed except in accordance with the terms of that
# agreement.
#
# Copyright (c) 2000-2003 Dell Inc. All Rights Reserved.
#
# Module Name:
#
#   iomsachk.sh
#
# Abstract/Purpose:
#
#   Shell script to check Server Administrator RPM dependencies
#
# Environment:
#
#   Linux
#
###############################################################################

PATH=./:/sbin:/usr/sbin:/bin:/usr/bin:${PATH}

SCRIPT_NAME=`echo $0 | sed s!^./!!`
STATUS_FILE="$1"

SA_PACKAGE_NAME="ServerAdministrator"
SAIS_PACKAGE_NAME="dellomsa"
SADD_PACKAGE_NAME="dellomsa-drivers"
SNMP_PACKAGE_NAME_UCD="ucd-snmp"
SNMP_PACKAGE_NAME_NET="net-snmp"

SA_RPM_NAME="${SA_PACKAGE_NAME}-1.0-0"
SA_RPM_FILE_NAME="${SA_PACKAGE_NAME}-1.0-0.i386.rpm"
SAIS_RPM_NAME="${SAIS_PACKAGE_NAME}-4.120-3924"
SAIS_RPM_FILE_NAME="${SAIS_PACKAGE_NAME}-4.120-3924.i386.rpm"
SADD_RPM_NAME="${SADD_PACKAGE_NAME}-4.120-3924"
SADD_RPM_FILE_NAME="${SADD_PACKAGE_NAME}-4.120-3924.i386.rpm"

LOG_FILE="/var/tmp/iomsachk.log"
TMP_FILE="/var/tmp/iomsachk.tmp"

KEY_PRODUCT_ID="Instrumentation"
KEY_STATUS="status"
KEY_NUMMSGS="numofmsgs"
KEY_MSGID="msgid"
KEY_INST_VER="installedversion"

VALUE_INST_VER_NONE="null"

MSG_ID_SYSTEM_NOT_SUPPORTED=1
MSG_ID_OMSA_INSTALLED_VER=15
MSG_ID_NO_OS_SNMP_SUPPORT=31
MSG_ID_NO_KERNEL_SOURCE=50
MSG_ID_KERNEL_NOT_SUPPORTED=51
MSG_ID_SNMP_PORT_CLOSED=53
MSG_ID_UNEXPECTED_ERROR=54
MSG_ID_INCOMPATIBLE_E100_DRIVER=55
MSG_ID_INCOMPATIBLE_NET_SNMP_PACKAGE=56

DEP_STATUS_SUCCESS=0
DEP_STATUS_WARNING=1
DEP_STATUS_ERROR=2

DEP_STATUS=${DEP_STATUS_SUCCESS}
MSG_COUNT=0
MSG_ID_LIST[0]=
MSG_ARGS_LIST[0]=


###############################################################################
#
#   Syntax:   iomsachk_display_msg <message>
#   Function: Display a message with the script name
#
###############################################################################
function iomsachk_display_msg
{
	echo "${SCRIPT_NAME}: $1"
}


###############################################################################
#
#   Syntax:   iomsachk_update_status <status code> <message ID> <message args>
#   Function: Update status for new message
#
###############################################################################
function iomsachk_update_status
{
	MSG_STATUS=$1
	MSG_ID=$2
	MSG_ARGS=$3

	# Change the status if it's worse than the current status
	if [ ${MSG_STATUS} -gt ${DEP_STATUS} ];
	then
		DEP_STATUS=${MSG_STATUS}
	fi

	# Add the message info to the lists
	MSG_ID_LIST[${MSG_COUNT}]=${MSG_ID}
	if [ ! -z ${MSG_ARGS} ];
	then
		MSG_ARGS_LIST[${MSG_COUNT}]=";${MSG_ARGS}"
	fi
	MSG_COUNT=$((MSG_COUNT + 1))
}


###############################################################################
#
#   Syntax:   iomsachk_write_status <key> <value>
#   Function: Write status information to the status file
#
###############################################################################
function iomsachk_write_status
{
	# Add product ID before writing to the file
	echo "${KEY_PRODUCT_ID}.$1=$2" >>${STATUS_FILE}
}


#------------------------------------------------------------------------------
# Perform sanity checks
#------------------------------------------------------------------------------

rm -f ${LOG_FILE}
rm -f ${TMP_FILE}

# Make sure we have the status file
if [ -z ${STATUS_FILE} ];
then
	iomsachk_display_msg "Status file not specified"
	exit 1
fi

# If the caller has set CDROM_DIR to a valid directory, go to that directory
if [ ! -z ${CDROM_DIR} ];
then
	if [ -d ${CDROM_DIR} ];
	then
		cd ${CDROM_DIR}
	fi
fi

# Make sure we have the RPMs
if [ ! -f ${SADD_RPM_FILE_NAME} ];
then
	iomsachk_display_msg "${SADD_RPM_FILE_NAME} not found"
	exit 2
fi
if [ ! -f ${SAIS_RPM_FILE_NAME} ];
then
	iomsachk_display_msg "${SAIS_RPM_FILE_NAME} not found"
	exit 3
fi
if [ ! -f ${SA_RPM_FILE_NAME} ];
then
	iomsachk_display_msg "${SA_RPM_FILE_NAME} not found"
	exit 4
fi

# Get list of currently installed RPMs
rpm -qa >${TMP_FILE} 2>&1

# Get installed version (if any); try Instrumentation first
VALUE_INST_VER=`
    grep ${SAIS_PACKAGE_NAME} ${TMP_FILE} |
    grep -v ${SADD_PACKAGE_NAME} |
    sed 's/-/ /g' |
    awk '{print $2}' |
    sed "s/\([0-9]\)\([.]\)\([0-9].*\)\([0-9]\)$/\1\2\3.\4/g"`
if [ -z ${VALUE_INST_VER} ];
then
	# No Instrumentation; try Device Drivers
	VALUE_INST_VER=`
        grep ${SADD_PACKAGE_NAME} ${TMP_FILE} |
        sed 's/-/ /g' |
        awk '{print $3}' |
        sed "s/\([0-9]\)\([.]\)\([0-9].*\)\([0-9]\)$/\1\2\3.\4/g"`
	if [ -z ${VALUE_INST_VER} ];
	then
		# No Instrumentation and no Device Drivers
		VALUE_INST_VER=${VALUE_INST_VER_NONE}
	fi
fi

#------------------------------------------------------------------------------
# Error checks
#------------------------------------------------------------------------------

# See if RPM with incompatible e100 driver is installed
if grep "^e100-1.6.14-1" ${TMP_FILE} >/dev/null   ||
   grep "^e100-1.6.14.1-1" ${TMP_FILE} >/dev/null ;
then
	# Update status for error condition
	iomsachk_update_status ${DEP_STATUS_ERROR} ${MSG_ID_INCOMPATIBLE_E100_DRIVER}
fi

# Do the machine and DKS checks only if Device Drivers RPM is not installed
grep ${SADD_PACKAGE_NAME} ${TMP_FILE} >/dev/null
if [ $? != 0 ];
then
	# Device Drivers RPM is not installed

	# Install the Device Drivers RPM to perform machine check and
	# DKS dependencies check
	#
	# We catch the RPM output so that we can get any error code because
	# rpm doesn't return our error code.  This relies on the error code
	# being echoed by our RPM and preceded by "ERROR ".
	rpm -i ${SADD_RPM_FILE_NAME} >${LOG_FILE} 2>&1
	if [ $? != 0 ];
	then
		# Get Device Drivers RPM error code
		SADD_ERROR=`grep "ERROR" ${LOG_FILE} | sed 's/.*ERROR //'`
		case "${SADD_ERROR}"
		in
			40) MSG_ID=${MSG_ID_SYSTEM_NOT_SUPPORTED} ;;

			41) MSG_ID=${MSG_ID_NO_KERNEL_SOURCE} ;;

			42) MSG_ID=${MSG_ID_KERNEL_NOT_SUPPORTED} ;;

			43) MSG_ID=${MSG_ID_KERNEL_NOT_SUPPORTED} ;;

			44) MSG_ID=${MSG_ID_KERNEL_NOT_SUPPORTED} ;;

			* ) MSG_ID="${MSG_ID_UNEXPECTED_ERROR};${LOG_FILE}" ;;
		esac

		# Update status for error condition
		iomsachk_update_status ${DEP_STATUS_ERROR} ${MSG_ID}
	else
		# Don't save the log file if no error
		rm -f ${LOG_FILE}
	fi

	# Remove the Device Drivers RPM
	rpm -e ${SADD_RPM_NAME} >/dev/null 2>&1
fi

#------------------------------------------------------------------------------
# Warning checks
#------------------------------------------------------------------------------

# See if the SNMP agent is installed
CHECK_SNMP_PORT=0
grep ${SNMP_PACKAGE_NAME_UCD} ${TMP_FILE} >/dev/null
if [ $? != 0 ];
then
	grep ${SNMP_PACKAGE_NAME_NET} ${TMP_FILE} >/dev/null
	if [ $? != 0 ];
	then
		# Update status for warning condition
		iomsachk_update_status ${DEP_STATUS_WARNING} ${MSG_ID_NO_OS_SNMP_SUPPORT}
	else
		CHECK_SNMP_PORT=1
	fi
else
	CHECK_SNMP_PORT=1
fi

# See if the firewall has the SNMP port closed
if [ CHECK_SNMP_PORT = 1 ];
then
	dcfwsnmp.sh
	if [ $? != 0 ];
	then
		# Update status for warning condition
		iomsachk_update_status ${DEP_STATUS_WARNING} ${MSG_ID_SNMP_PORT_CLOSED}
	fi
fi

# See if buggy net-snmp is installed
if grep "^net-snmp-5.0.1-6" ${TMP_FILE} >/dev/null 
then
	# Update status for warning condition
	iomsachk_update_status ${DEP_STATUS_WARNING} ${MSG_ID_INCOMPATIBLE_NET_SNMP_PACKAGE}
fi


#------------------------------------------------------------------------------
# Information checks
#------------------------------------------------------------------------------

# See if Instrumentation is installed
if [ "${VALUE_INST_VER}" != "${VALUE_INST_VER_NONE}" ];
then
	INST_IS_VER=`echo ${VALUE_INST_VER} | sed "s/\([0-9]\)\([.]\)\([0-9].*\)\(.\)\([0-9].*\)/\1\3\5/g"`
	CUR_IS_VER=`echo ${SAIS_RPM_NAME} | sed "s/[a-z]*//g" |sed "s/\(-\)\([0-9]\)\(.\)\([0-9].*\)\(-.*\)/\2\4/g"`

	# Update status for information condition
	if [ ${INST_IS_VER} = ${CUR_IS_VER} ];
	then
		iomsachk_update_status ${DEP_STATUS_WARNING} ${MSG_ID_OMSA_INSTALLED_VER} ${VALUE_INST_VER}
	fi
	if [ ${INST_IS_VER} -lt ${CUR_IS_VER} ];
	then
		iomsachk_update_status ${DEP_STATUS_SUCCESS} ${MSG_ID_OMSA_INSTALLED_VER} ${VALUE_INST_VER}
	fi
	if [ ${INST_IS_VER} -gt ${CUR_IS_VER} ];
	then
		iomsachk_update_status ${DEP_STATUS_ERROR} ${MSG_ID_OMSA_INSTALLED_VER} ${VALUE_INST_VER}
	fi
fi

#------------------------------------------------------------------------------
# Ouput status to the status file
#------------------------------------------------------------------------------
iomsachk_write_status ${KEY_STATUS} ${DEP_STATUS}
iomsachk_write_status ${KEY_NUMMSGS} ${MSG_COUNT}
TMP_COUNT=0
while [ ${TMP_COUNT} -lt ${MSG_COUNT} ];
do
	iomsachk_write_status "${KEY_MSGID}.${TMP_COUNT}" "${MSG_ID_LIST[${TMP_COUNT}]}${MSG_ARGS_LIST[${TMP_COUNT}]}"
	TMP_COUNT=$((TMP_COUNT + 1))
done
iomsachk_write_status ${KEY_INST_VER} ${VALUE_INST_VER}

#------------------------------------------------------------------------------
# The End
#------------------------------------------------------------------------------
rm -f ${TMP_FILE}
exit 0

